;=============================================================
; Měření frekvence procesoru v PMD85
; - změří frekvenci signálu FI2 (2,048MHz/2,000MHz), tu pak
;   násobí 9
; - využívá čítač/časovač T2, který je taktován 1Hz
;
; Autor: Zdeněk Šesták, 2018
; Web:   maximalne.8u.cz
;=============================================================
                  org   0
                  jmp   main
; funkce závislé na monitoru
sys_print:        jmp   8500h
sys_exit:         jmp   803eh
; buffer pro číslo převedené na text
numbuffer:        ds    6
; hlavní program
main:             lxi   sp, 7ffeh
                  lxi   h,txtTitle
                  call  prttext
                  mvi   a, 90h          ; T2/lsb/Mode0/binary
                  out   5fh
                  mvi   a, 255
                  out   5eh
                  ; čeká na 0feh
                  lxi   d,0             ; 12T nulování počítadla
                  mvi   b,0feh
.wait:            in    5eh
                  cmp   b
                  jnz   .wait
                  mov   b,a
.m:               ; smyčka trvá přesně 1000 taktů
                  mvi   l,22            ; 8T
                  
.delay:           push  psw             ; 12
                  pop   psw             ; 12
                  dcr   l               ; 6
                  jnz   .delay          ; 12 = 42T * 22 = 924T

                  in    0f5h            ; 12T
                  add   a               ; 4T
                  jp    sys_exit        ; 12T = 28T

                  mov   a,a             ; 6T
                  inx   d               ; 6T
                  in    5eh             ; 12T
                  cmp   b               ; 4T
                  jz    .m              ; 12T = 40T
                  ; ----------------
                  ; 8T + 924T + 28T + 40T = 1000T
                  ; DE*9
                  mov   h,d
                  mov   l,e
                  dad   h
                  dad   h
                  dad   h
                  dad   d
                  ; HL=>string
                  call  i2string
                  ; zobrazí číslo s desetinnou tečkou za druhou číslicí
                  call  prtnum
                  ; příprava na spuštění měření
                  lxi   d,0             ; nulování počítadla
                  in    5eh
                  mov   b,a
                  dcr   b
                  jmp   .wait
; převod 16bitového celého čísla na posloupnost znaků
; vstup: HL - číslo
; výstup: numbuffer - nulou zakončený řetězec
i2string:         lxi   b,1000h
                  lxi   d, 0
.loop:            dad   h
                  mov   a,e
                  adc   a
                  daa
                  mov   e,a
                  mov   a,d
                  adc   a
                  daa
                  mov   d,a
                  mov   a,c
                  adc   a
                  daa
                  mov   c,a
                  dcr   b
                  jnz   .loop
                  lxi   h, numbuffer
.po:              ; test první číslice v registru C
                  xra   a
                  ora   c
                  jz    .next
                  adi   30h
                  mov   m,a
                  inx   h
                  mvi   c,1
.next:            mov   a,d
                  call  .uplow
                  mov   a,e
                  call  .uplow
                  xra   a
                  ora   c
                  jnz   .nonzero
                  mvi   m,'0'           ; číslo je 0
                  inx   h
.nonzero:         mvi   m,0
                  ret
.uplow:           push  psw
                  call  .upper
                  pop   psw
                  jmp   .lower
.upper:           rrc
                  rrc
                  rrc
                  rrc
.lower:           ani   0fh
                  jnz   .nonz
                  inr   c
                  dcr   c
                  rz                    ; nuly na začátku nezapisujeme
.nonz:            adi   30h
                  inr   c
                  mov   m,a
                  inx   h
                  ret
prtnum:           lxi   h, numbuffer
                  mov   a,m
                  inx   h
                  call  sys_print
                  mov   a,m
                  inx   h
                  call  sys_print
                  mvi   a,'.'
                  call  sys_print
                  call  prttext
                  lxi   h, txtMhz
prttext:          mov   a,m
                  inx   h
                  ana   a
                  rz
                  call  sys_print
                  jmp   prttext
txtMhz:           db    " MHZ", 0dh,0
txtTitle:         db    01CH, "MERENI FREKVENCE PROCESORU",0dH
                  db    "==========================",0dH
                  db    "KONEC PROGRAMU........STOP",0dH,0